@@ -48,6 +48,11 @@
 #include "Preference.h"
 
 
+#define ENABLE_HELPER  1
+
+//#define DEBUG_IM  1
+
+
 const short kMENU_Pencil = kBaseResourceID + 1;
 
 const short kICNx_Pencil = kBaseResourceID + 1;
@@ -78,7 +83,14 @@ enum
   kHideSendEventPaletteMenuItemString = 4
 };
 
-static MUIMSessionHandle gActiveSession;
+MUIMSessionHandle gActiveSession;
+Boolean gDisableFocusedContext = FALSE;
+
+int gUimFD;
+CFSocketRef gUimSock;
+CFRunLoopSourceRef gUimRun;
+CFSocketContext gSockContext;
+
 static MenuRef gPencilMenu;
 
 static CFStringRef gShowKeyboardPaletteMenuItemString;
@@ -86,8 +98,6 @@ static CFStringRef gHideKeyboardPaletteMenuItemString;
 static CFStringRef gShowSendEventPaletteMenuItemString;
 static CFStringRef gHideSendEventPaletteMenuItemString;
 
-extern SInt32 uimFD;
-
 static pascal OSStatus MUIMPencilMenuEventHandler(EventHandlerCallRef
                                                  inEventHandlerCallRef,
                                                  EventRef inEventRef,
@@ -114,6 +124,7 @@ MUIMInitialize(ComponentInstance inComponentInstance,
 
   gActiveSession = NULL;
   gPencilMenu = NULL;
+  gDisableFocusedContext = FALSE;
 
   refNum = OpenComponentResFile((Component) inComponentInstance);
   result = ResError();
@@ -208,6 +219,30 @@ MUIMInitialize(ComponentInstance inComponentInstance,
     result = MUIMInitializeMessageReceiving();
   */
 
+  DEBUG_PRINT("MUIMInitialize()\n");
+
+  {
+    char *home = getenv("HOME");
+    if (home == NULL) {
+      struct passwd *pw = NULL;
+      uid_t uid = getuid();
+      pw = getpwuid(uid);
+      if (pw == NULL)
+        setenv("HOME", "/", 0);
+      else
+        setenv("HOME", pw->pw_dir, 0);
+    }
+  }
+
+  // for Uim debug
+  //setenv("LIBUIM_VERBOSE", "5", 1);
+
+  uim_init();
+
+  gUimFD = -1;
+  gUimSock = NULL;
+  gUimRun = NULL;
+
   return result;
 }
 
@@ -216,7 +251,14 @@ MUIMTerminate(ComponentInstance inComponentInstance)
 {
     DEBUG_PRINT("MUIMTerminate()\n");
 
+#ifdef ENABLE_HELPER
+    UIMHelperClose();
+#endif
+
+    uim_quit();
+
     gActiveSession = NULL;
+    gDisableFocusedContext = FALSE;
     gPencilMenu = NULL;
 }
 
@@ -244,32 +286,18 @@ MUIMSessionOpen(ComponentInstance inComponentInstance,
 
   DEBUG_PRINT("MUIMSessionOpen() imName='%s'\n", imName);
 
-  if (*outSessionHandle == nil)
+  if (*outSessionHandle == nil) {
     *outSessionHandle =
       (MUIMSessionHandle) NewHandle(sizeof(MUIMSessionRecord));
+    DEBUG_PRINT("MUIMSessionOpen() NewHandle=%p\n", *outSessionHandle);
+  }
 
   if (*outSessionHandle) {
-    char *home = getenv("HOME");
-    if (home == NULL) {
-      struct passwd *pw = NULL;
-      uid_t uid = getuid();
-      pw = getpwuid(uid);
-      if (pw == NULL)
-        setenv("HOME", "/", 0);
-      else
-        setenv("HOME", pw->pw_dir, 0);
-    }
-
-    // for Uim debug
-    //setenv("LIBUIM_VERBOSE", "5", 1);
-
     (**outSessionHandle)->fComponentInstance = inComponentInstance;
     (**outSessionHandle)->fLastUpdateLength = 0;
     (**outSessionHandle)->fSegments = NULL;
     (**outSessionHandle)->fSegmentCount = 0;
 
-    uim_init();
-
     (**outSessionHandle)->fUC =
       uim_create_context(*outSessionHandle, "UTF-8", NULL, imName,
                          uim_iconv, UIMCommitString);
@@ -292,11 +320,22 @@ MUIMSessionOpen(ComponentInstance inComponentInstance,
     }
 #endif
 
-    uim_set_mode_cb((**outSessionHandle)->fUC, UIMMode);
+#ifdef ENABLE_HELPER
+    UIMCheckHelper(*outSessionHandle);
+#endif
+
     uim_set_preedit_cb((**outSessionHandle)->fUC,
                        UIMPreeditClear,
                        UIMPreeditPushback,
                        UIMPreeditUpdate);
+#ifdef ENABLE_HELPER
+    uim_set_prop_list_update_cb((**outSessionHandle)->fUC,
+                                UIMUpdatePropList);
+    uim_set_prop_label_update_cb((**outSessionHandle)->fUC,
+                                 UIMUpdatePropLabel);
+
+    uim_prop_list_update((**outSessionHandle)->fUC);
+#endif
     uim_set_candidate_selector_cb((**outSessionHandle)->fUC,
                                   UIMCandAcivate,
                                   UIMCandSelect,
@@ -348,6 +387,7 @@ MUIMSessionActivate(MUIMSessionHandle inSessionHandle)
   OSStatus result = noErr;
 
   gActiveSession = inSessionHandle;
+  gDisableFocusedContext = FALSE;
 
   long keyboardID = GetScriptVariable(GetScriptManagerVariable(smLastScript),
                                       smScriptKeys);
@@ -356,6 +396,15 @@ MUIMSessionActivate(MUIMSessionHandle inSessionHandle)
   DEBUG_PRINT("MUIMSessionActivate() gActiveSession=%p\n",
               gActiveSession);
 
+#ifdef ENABLE_HELPER
+  UIMCheckHelper(inSessionHandle);
+
+  uim_prop_list_update((*inSessionHandle)->fUC);
+  uim_prop_label_update((*inSessionHandle)->fUC);
+
+  uim_helper_client_focus_in((*inSessionHandle)->fUC);
+#endif
+
   return result;
 }
 
@@ -365,7 +414,16 @@ MUIMSessionDeactivate(MUIMSessionHandle inSessionHandle)
   DEBUG_PRINT("MUIMSessionDeactivate() gActiveSession=%p\n",
               gActiveSession);
 
-  gActiveSession = nil;
+#ifdef ENABLE_HELPER
+  uim_helper_client_focus_out((*inSessionHandle)->fUC);
+#endif
+
+  /*
+  if (inSessionHandle == gActiveSession) {
+    gActiveSession = nil;
+    gDisableFocusedContext = TRUE;
+  }
+  */
 
   return noErr;
 }
@@ -389,8 +447,8 @@ MUIMSessionEvent(MUIMSessionHandle inSessionHandle, EventRef inEventRef)
   // kEventHotKeyPressed = 5
   // kEventHotKeyReleased = 6
 
-  DEBUG_PRINT("MUIMSessionEvent() eventClass=%lu eventKind=%lu\n",
-              eventClass, eventKind);
+  //DEBUG_PRINT("MUIMSessionEvent() eventClass=%lu eventKind=%lu\n",
+  //            eventClass, eventKind);
 
   // kEventClassKeyboard:
   //   kEventRawKeyDown - A key was pressed
@@ -417,7 +475,8 @@ MUIMSessionEvent(MUIMSessionHandle inSessionHandle, EventRef inEventRef)
     DEBUG_PRINT("MUIMSessionEvent() keycode=0x%lx, char=%c, charCode=0x%x, modifiers=0x%lx\n",
                 keyCode, charCode, charCode, modifiers);
 
-    handled = MUIMHandleInput(inSessionHandle, keyCode, charCode, modifiers);
+    if (!(modifiers & cmdKey))
+       handled = MUIMHandleInput(inSessionHandle, keyCode, charCode, modifiers);
   }
 
   return handled;
@@ -480,21 +539,14 @@ MUIMHandleInput(MUIMSessionHandle inSessionHandle, UInt32 inKeycode,
   DEBUG_PRINT("MUIMHandleInput() key=0x%x mod=0x%x\n", key, mod);
 
   rv = uim_press_key((*inSessionHandle)->fUC, key, mod);
-  DEBUG_PRINT("MUIMHandleInput() uim_press_key ret=%d\n", rv);
 
   uim_release_key((*inSessionHandle)->fUC, key, mod);
 
-  if (!rv || isScriptKey) {
+  if (!rv || isScriptKey)
     handled = TRUE;
-    /*
-    if ((*inSessionHandle)->fFixLen > 0) {
-      MUIMUpdateActiveInputArea(inSessionHandle, TRUE);
-      free((*inSessionHandle)->fFixBuffer);
-      (*inSessionHandle)->fFixBuffer = NULL;
-      (*inSessionHandle)->fFixLen = 0;
-    }
-    */
-  }
+
+  DEBUG_PRINT("MUIMHandleInput() uim_press_key handled=%s\n",
+              handled ? "true" : "false");
 
   return handled;
 }
@@ -587,9 +639,9 @@ ConvertModifier(int inMod)
 
   DEBUG_PRINT("ConvertModifier() inMod=%lx\n", inMod);
 
-  if (inMod & 0x200)
+  if (inMod & shiftKey)
     modifier += UMod_Shift;
-  if (inMod & 0x1000)
+  if (inMod & controlKey)
     modifier += UMod_Control;
 
   return modifier;
