@@ -42,6 +42,8 @@
 pascal ComponentResult MUIMComponentDispatch(ComponentParameters * inParams,
                                              Handle inSessionHandle);
 
+extern MUIMSessionHandle gActiveSession;
+
 long gInstanceRefCount = 0;
 MenuRef gTextServiceMenu = nil;
 
@@ -192,6 +194,8 @@ MUIMOpenComponent(ComponentInstance inComponentInstance)
   ComponentResult result = noErr;
   Handle sessionHandle = nil;
 
+  DEBUG_PRINT("MUIMOpenComponent() count=%ld + 1\n", gInstanceRefCount);
+
   if (gInstanceRefCount == 0)
     result = MUIMInitialize(inComponentInstance, &gTextServiceMenu);
   gInstanceRefCount++;
@@ -199,9 +203,11 @@ MUIMOpenComponent(ComponentInstance inComponentInstance)
   if (result == noErr) {
     sessionHandle = GetComponentInstanceStorage(inComponentInstance);
 
-    result =
-      MUIMSessionOpen(inComponentInstance,
-                      (MUIMSessionHandle *) & sessionHandle);
+    //if (gInstanceRefCount == 1)
+    //  sessionHandle = (Handle) gActiveSession;
+    //else
+      result = MUIMSessionOpen(inComponentInstance,
+                               (MUIMSessionHandle *) &sessionHandle);
 
     if (result == noErr)
       SetComponentInstanceStorage(inComponentInstance, sessionHandle);
@@ -227,7 +233,7 @@ MUIMCloseComponent(Handle inSessionHandle, ComponentInstance inComponentInstance
 {
   ComponentResult result = noErr;
 
-  DEBUG_PRINT("MUIMCloseComponent()\n");
+  DEBUG_PRINT("MUIMCloseComponent() count=%d - 1\n", gInstanceRefCount);
 
   if (inComponentInstance == nil)
     result = paramErr;
