@@ -39,11 +39,26 @@
 #include "UIMCallback.h"
 #include "CandidateCarbon.h"
 
+extern MUIMSessionHandle gActiveSession;
+extern Boolean gDisableFocusedContext;
+
+extern int gUimFD;
+extern CFSocketRef gUimSock;
+extern CFRunLoopSourceRef gUimRun;
+extern CFSocketContext gSockContext;
+
 
 static void
 AddPreeditSegment(MUIMSessionHandle inHandle, int inAttr,
                   const char *inStr);
 
+static void
+HelperRead(CFSocketRef sock, CFSocketCallBackType callbackType, 
+           CFDataRef address, const void *data, void *info);
+
+static void
+ParseHelperString(const char *str);
+
 
 void
 UIMCommitString(void *ptr, const char *str)
@@ -79,19 +94,6 @@ UIMCommitString(void *ptr, const char *str)
   }
 }
 
-void
-UIMMode(void *ptr, int mode)
-{
-  MUIMSessionHandle handle = (MUIMSessionHandle) ptr;
-
-  DEBUG_PRINT("UIMMode() mode=%d oldmode=%d\n",
-              mode, (*handle)->fMode);
-
-  if ((*handle)->fMode == mode)
-    return;
-
-  (*handle)->fMode = mode;
-}
 
 void
 UIMPreeditClear(void *ptr)
@@ -241,6 +243,170 @@ UIMCandDeactivate(void *inPtr)
   HideCandidateWindow(handle);
 }
 
+void
+UIMUpdatePropList(void *inPtr, const char *inStr)
+{
+  char *tmp;
+
+  if (!gActiveSession) return;
+
+  tmp = (char *) malloc(sizeof(char) *
+                        (strlen(kPropListUpdate) + 1 +
+                         strlen(inStr) + 1));
+  if (tmp) {
+    if (gUimFD >= 0) {
+      snprintf(tmp, strlen(kPropListUpdate) + 1 + strlen(inStr) + 1,
+               "%s\n%s",
+               kPropListUpdate, inStr);
+      uim_helper_send_message(gUimFD, tmp);
+    }
+    free(tmp);
+  }
+}
+
+void
+UIMUpdatePropLabel(void *inPtr, const char *inStr)
+{
+  char *tmp;
+
+  if (!gActiveSession) return;
+
+  tmp = (char *) malloc(sizeof(char) *
+                        (strlen(kPropLabelUpdate) + 1 +
+                         strlen(inStr) + 1));
+  if (tmp) {
+    if (gUimFD >= 0) {
+      snprintf(tmp, strlen(kPropLabelUpdate) + 1 + strlen(inStr) + 1,
+               "%s\n%s",
+               kPropLabelUpdate, inStr);
+      uim_helper_send_message(gUimFD, tmp);
+    }
+    free(tmp);
+  }
+}
+
+void
+UIMCheckHelper(MUIMSessionHandle inHandle)
+{
+  if (gUimFD < 0) {
+    gUimFD = uim_helper_init_client_fd(UIMHelperDisconnect);
+    DEBUG_PRINT("UIMCheckHelper() gUimFD=%d(%p)\n", gUimFD, &gUimFD);
+
+    if (gUimFD >= 0) {
+      if (!gUimSock) {
+        gSockContext.version = 0;
+        gSockContext.info = inHandle;
+        gSockContext.retain = NULL;
+        gSockContext.release = NULL;
+        gSockContext.copyDescription = NULL;
+
+        gUimSock = CFSocketCreateWithNative(kCFAllocatorDefault, gUimFD,
+                                            kCFSocketReadCallBack, HelperRead,
+                                            &gSockContext);
+        if (!gUimSock) return;
+      }
+
+      if (!gUimRun) {
+        gUimRun = CFSocketCreateRunLoopSource(kCFAllocatorDefault, gUimSock, 0);
+        if (!gUimRun) {
+          CFRelease(gUimSock);
+          gUimSock = NULL;
+          return;
+        }
+        CFRunLoopAddSource(CFRunLoopGetCurrent(), gUimRun, kCFRunLoopDefaultMode);
+        DEBUG_PRINT("UIMCheckHelper() CFRunLoopGetCurrent()\n");
+      }
+    }
+  }
+}
+
+static void
+HelperRead(CFSocketRef sock, CFSocketCallBackType callbackType, 
+           CFDataRef address, const void *data, void *info)
+{
+  char *tmp;
+
+  DEBUG_PRINT("HelperRead()\n");
+
+  uim_helper_read_proc(CFSocketGetNative(sock));
+  while ((tmp = uim_helper_get_message())) {
+    ParseHelperString(tmp);
+    free(tmp);
+  }
+}
+
+static void
+ParseHelperString(const char *str)
+{
+  DEBUG_PRINT("ParseHelperString() str='%s'\n", str);
+
+  if (gActiveSession && !gDisableFocusedContext) {
+    if (strncmp("prop_list_get", str, 13) == 0)
+      uim_prop_list_update((*gActiveSession)->fUC);
+    else if (strncmp("prop_label_get", str, 14) == 0)
+      uim_prop_label_update((*gActiveSession)->fUC);
+    else if (strncmp("prop_activate", str, 13) == 0) {
+      CFMutableStringRef cfstr;
+      CFArrayRef array;
+      CFStringRef first;
+
+      cfstr = CFStringCreateMutable(NULL, 0);
+      CFStringAppendCString(cfstr, str, kCFStringEncodingUTF8);
+
+      array = CFStringCreateArrayBySeparatingStrings(kCFAllocatorDefault,
+                                                     cfstr, CFSTR("\n"));
+
+      if (array && CFArrayGetCount > 0 &&
+          (first = CFArrayGetValueAtIndex(array, 0))) {
+        CFStringRef second = CFArrayGetValueAtIndex(array, 0);
+        if (second) {
+          uim_prop_activate((*gActiveSession)->fUC, (char *) CFStringGetCharactersPtr(second));
+          CFRelease(second);
+        }
+        CFRelease(first);
+      }
+    }
+    else if (strncmp("focus_in", str, 8) == 0) {
+      //gDisableFocusedContext = TRUE;
+      /* We shouldn't do "focused_context = NULL" here, because some
+	 window manager has some focus related bugs. */
+    }
+  }
+}
+
+void
+UIMHelperDisconnect()
+{
+  DEBUG_PRINT("UIMHelperDisconnect()\n");
+
+  CFRunLoopRemoveSource(CFRunLoopGetCurrent(), gUimRun, kCFRunLoopDefaultMode);
+
+  CFRelease(gUimRun);
+  CFRelease(gUimSock);
+
+  gUimRun = NULL;
+  gUimSock = NULL;
+
+  gUimFD = -1;
+}
+
+void
+UIMHelperClose()
+{
+  DEBUG_PRINT("UIMHelperClose()\n");
+
+  //if (gUimRun) {
+  //  CFRunLoopRemoveSource(CFRunLoopGetCurrent(), gUimRun, kCFRunLoopDefaultMode);
+  //  CFRelease(gUimRun);
+  //}
+  
+  //if (gUimSock)
+  //  CFRelease(gUimSock);
+
+  if (gUimFD >= 0)
+    uim_helper_close_client_fd(gUimFD);
+}
+
 static void
 AddPreeditSegment(MUIMSessionHandle inHandle, int inAttr,
                   const char *inStr)
